=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2006-03-01 08:40:35 +0000
+++ bzrlib/commit.py	2006-03-02 06:23:15 +0000
@@ -422,7 +422,7 @@
         for path, ie in self.new_inv.iter_entries():
             previous_entries = ie.find_previous_heads(
                 self.parent_invs, 
-                self.weave_store.get_weave_prelude_or_empty(ie.file_id,
+                self.weave_store.get_weave_or_empty(ie.file_id,
                     self.branch.get_transaction()))
             if ie.revision is None:
                 change = ie.snapshot(self.rev_id, path, previous_entries,

=== modified file 'bzrlib/fetch.py'
--- bzrlib/fetch.py	2006-03-01 12:09:59 +0000
+++ bzrlib/fetch.py	2006-03-02 06:23:15 +0000
@@ -168,7 +168,8 @@
                 to_weave.join(from_weave)
             else:
                 # destination is empty, just replace it
-                self.to_weaves.copy_multi(self.from_weaves, [file_id], self.pb)
+                self.to_weaves.copy_multi(self.from_weaves, [file_id], self.pb,
+                                          self.from_repository.get_transaction())
         self.pb.clear()
 
     def _fetch_inventory_weave(self, revs):

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2006-03-01 08:40:35 +0000
+++ bzrlib/inventory.py	2006-03-02 06:23:15 +0000
@@ -495,7 +495,7 @@
             w.check()
             checker.checked_weaves[self.file_id] = True
         else:
-            w = tree.get_weave_prelude(self.file_id)
+            w = tree.get_weave(self.file_id)
 
         mutter('check version {%s} of {%s}', rev_id, self.file_id)
         checker.checked_text_cnt += 1 

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2006-03-02 01:26:22 +0000
+++ bzrlib/repository.py	2006-03-02 06:23:15 +0000
@@ -27,7 +27,6 @@
 from bzrlib.lockable_files import LockableFiles
 from bzrlib.osutils import safe_unicode
 from bzrlib.revision import NULL_REVISION
-from bzrlib.store import copy_all
 from bzrlib.store.versioned.weave import WeaveStore
 from bzrlib.store.text import TextStore
 from bzrlib.symbol_versioning import *
@@ -1131,13 +1130,17 @@
             # FIXME do not peek!
             if self.source.control_files._transport.listable():
                 pb = bzrlib.ui.ui_factory.progress_bar()
-                copy_all(self.source.weave_store,
-                    self.target.weave_store, pb=pb)
+                self.target.weave_store.copy_all_ids(
+                    self.source.weave_store,
+                    pb=pb,
+                    from_transaction=self.source.get_transaction())
                 pb.update('copying inventory', 0, 1)
                 self.target.control_weaves.copy_multi(
-                    self.source.control_weaves, ['inventory'])
-                copy_all(self.source.revision_store,
-                    self.target.revision_store, pb=pb)
+                    self.source.control_weaves, ['inventory'],
+                    from_transaction=self.source.get_transaction())
+                self.target.revision_store.copy_all_ids(
+                    self.source.revision_store,
+                    pb=pb)
             else:
                 self.target.fetch(self.source, revision_id=revision_id)
 

=== modified file 'bzrlib/store/__init__.py'
--- bzrlib/store/__init__.py	2006-02-17 02:01:02 +0000
+++ bzrlib/store/__init__.py	2006-03-02 06:23:15 +0000
@@ -32,6 +32,7 @@
 import bzrlib
 import bzrlib.errors as errors
 from bzrlib.errors import BzrError, UnlistableStore, TransportNotPossible
+from bzrlib.symbol_versioning import *
 from bzrlib.trace import mutter
 import bzrlib.transport as transport
 from bzrlib.transport.local import LocalTransport
@@ -87,6 +88,20 @@
         """Return True if this store is able to be listed."""
         return hasattr(self, "__iter__")
 
+    def copy_all_ids(self, store_from, pb=None):
+        """Copy all the file ids from store_from into self."""
+        if not store_from.listable():
+            raise UnlistableStore(store_from)
+        ids = []
+        for count, file_id in enumerate(store_from):
+            if pb:
+                pb.update('listing files', count, count)
+            ids.append(file_id)
+        if pb:
+            pb.clear()
+        mutter('copy_all ids: %r', ids)
+        self.copy_multi(store_from, ids, pb=pb)
+
     def copy_multi(self, other, ids, pb=None, permit_failure=False):
         """Copy texts for ids from other into self.
 
@@ -312,20 +327,10 @@
     return bzrlib.store.text.TextStore(transport.memory.MemoryTransport())
         
 
+@deprecated_function(zero_eight)
 def copy_all(store_from, store_to, pb=None):
     """Copy all ids from one store to another."""
-    # TODO: Optional progress indicator
-    if not store_from.listable():
-        raise UnlistableStore(store_from)
-    ids = []
-    for count, file_id in enumerate(store_from):
-        if pb:
-            pb.update('listing files', count, count)
-        ids.append(file_id)
-    if pb:
-        pb.clear()
-    mutter('copy_all ids: %r', ids)
-    store_to.copy_multi(store_from, ids, pb=pb)
+    store_to.copy_all_ids(store_from, pb)
 
 
 def hash_prefix(fileid):

=== modified file 'bzrlib/store/versioned/weave.py'
--- bzrlib/store/versioned/weave.py	2006-03-01 08:40:35 +0000
+++ bzrlib/store/versioned/weave.py	2006-03-02 06:23:15 +0000
@@ -26,7 +26,7 @@
 import urllib
 
 from bzrlib.weavefile import read_weave, write_weave_v5
-from bzrlib.weave import Weave, WeaveFile
+from bzrlib.weave import WeaveFile
 from bzrlib.store import TransportStore, hash_prefix
 from bzrlib.atomicfile import AtomicFile
 from bzrlib.errors import NoSuchFile, FileExists
@@ -67,7 +67,10 @@
     def _get(self, file_id):
         return self._transport.get(self.filename(file_id))
 
-    def _put(self, file_id, f):
+    def _put(self, file_id, weave):
+        f = StringIO()
+        write_weave_v5(weave, f)
+        f.seek(0)
         # less round trips to mkdir on failure than mkdir always
         try:
             return self._transport.put(self.filename(file_id), f, mode=self._file_mode)
@@ -85,22 +88,6 @@
         w = WeaveFile(self.filename(file_id), self._transport, self._file_mode)
         transaction.map.add_weave(file_id, w)
         transaction.register_clean(w, precious=self._precious)
-        # TODO: jam 20051219 This should check if there is a prelude
-        #       which is already cached, and if so, should remove it
-        #       But transaction doesn't seem to have a 'remove'
-        #       One workaround would be to re-add the object with
-        #       the PRELUDE marker.
-        return w
-
-    def get_weave_prelude(self, file_id, transaction):
-        weave_id = file_id
-        weave = transaction.map.find_weave(weave_id)
-        if weave:
-            mutter("cache hit in %s for %s", self, weave_id)
-            return weave
-        w = read_weave(self._get(file_id), prelude=True)
-        # no point caching the prelude: any repeat action will need the real 
-        # thing
         return w
 
     def get_lines(self, file_id, rev_id, transaction):
@@ -110,26 +97,24 @@
         w = self.get_weave(file_id, transaction)
         return w.get(rev_id)
     
-    def get_weave_prelude_or_empty(self, file_id, transaction):
-        """cheap version that reads the prelude but not the lines
-        """
-        try:
-            return self.get_weave_prelude(file_id, transaction)
-        except NoSuchFile:
-            # returns more than needed - harmless as its empty.
-            return self._new_weave(file_id, transaction)
-
     def _new_weave(self, file_id, transaction):
         """Make a new weave for file_id and return it."""
-        weave = WeaveFile(self.filename(file_id), self._transport, self._file_mode)
-        # ensure that the directories are created.
-        # this is so that weave does not encounter ENOTDIR etc.
-        weave_stream = self._weave_to_stream(weave)
-        self._put(file_id, weave_stream)
+        weave = self._make_new_versionedfile(file_id)
         transaction.map.add_weave(file_id, weave)
         transaction.register_clean(weave, precious=self._precious)
         return weave
 
+    def _make_new_versionedfile(self, file_id):
+        try:
+            weave = WeaveFile(self.filename(file_id), self._transport, self._file_mode)
+        except NoSuchFile:
+            if not self._prefixed:
+                # unexpected error - NoSuchFile is raised on a missing dir only.
+                raise
+            self._transport.mkdir(hash_prefix(file_id), mode=self._dir_mode)
+            weave = WeaveFile(self.filename(file_id), self._transport, self._file_mode)
+        return weave
+
     def get_weave_or_empty(self, file_id, transaction):
         """Return a weave, or an empty one if it doesn't exist.""" 
         try:
@@ -151,16 +136,7 @@
 
     def _put_weave(self, file_id, weave, transaction):
         """Preserved here for upgrades-to-weaves to use."""
-        transaction.register_dirty(weave)
-        weave_stream = self._weave_to_stream(weave)
-        self._put(file_id, weave_stream)
-
-    def _weave_to_stream(self, weave):
-        """Make a stream from a weave."""
-        sio = StringIO()
-        write_weave_v5(weave, sio)
-        sio.seek(0)
-        return sio
+        self._put(file_id, weave)
 
     @deprecated_method(zero_eight)
     def add_text(self, file_id, rev_id, new_lines, parents, transaction):
@@ -183,8 +159,35 @@
         vfile = self.get_weave_or_empty(file_id, transaction)
         vfile.clone_text(new_rev_id, old_rev_id, parents)
      
+    def copy_all_ids(self, store_from, pb=None, from_transaction=None):
+        """Copy all the file ids from store_from into self."""
+        if from_transaction is None:
+            warn("Please pase from_transaction into "
+                 "versioned_store.copy_all_ids.", stacklevel=2)
+        if not store_from.listable():
+            raise UnlistableStore(store_from)
+        ids = []
+        for count, file_id in enumerate(store_from):
+            if pb:
+                pb.update('listing files', count, count)
+            ids.append(file_id)
+        if pb:
+            pb.clear()
+        mutter('copy_all ids: %r', ids)
+        self.copy_multi(store_from, ids, pb=pb,
+                        from_transaction=from_transaction)
+
     def copy_multi(self, from_store, file_ids, pb=None, from_transaction=None):
+        """Copy all the versions for multiple file_ids from from_store.
+        
+        :param from_transaction: required current transaction in from_store.
+        """
         assert isinstance(from_store, WeaveStore)
+        if from_transaction is None:
+            warn("WeaveStore.copy_multi without a from_transaction parameter "
+                 "is deprecated. Please provide a from_transaction.",
+                 DeprecationWarning,
+                 stacklevel=2)
         for count, f in enumerate(file_ids):
             mutter("copy weave {%s} into %s", f, self)
             if pb:
@@ -193,9 +196,12 @@
             if from_transaction and from_transaction.map.find_weave(f):
                 mutter("cache hit in %s for %s", from_store, f)
                 weave = from_transaction.map.find_weave(f)
-                weave_stream = self._weave_to_stream(weave)
-                self._put(f, weave_stream)
+                # joining is fast with knits, and bearable for weaves -
+                # indeed the new case can be optimised
+                target = self._make_new_versionedfile(f)
+                target.join(weave)
             else:
-                self._put(f, from_store._get(f))
+                from bzrlib.transactions import PassThroughTransaction
+                self._put(f, from_store.get_weave(f, PassThroughTransaction()))
         if pb:
             pb.clear()

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2006-03-02 03:12:34 +0000
+++ bzrlib/tree.py	2006-03-02 06:23:15 +0000
@@ -132,10 +132,6 @@
         return self._weave_store.get_weave(file_id,
                 self._branch.get_transaction())
 
-    def get_weave_prelude(self, file_id):
-        return self._weave_store.get_weave_prelude(file_id,
-                self._branch.get_transaction())
-
     def get_file_lines(self, file_id):
         ie = self._inventory[file_id]
         weave = self.get_weave(file_id)

=== modified file 'bzrlib/weave.py'
--- bzrlib/weave.py	2006-03-02 03:12:34 +0000
+++ bzrlib/weave.py	2006-03-02 06:23:15 +0000
@@ -831,8 +831,8 @@
         try:
             _read_weave_v5(self._transport.get(name), self)
         except errors.NoSuchFile:
-            # new file, no-op.
-            pass
+            # new file, save it
+            self._save()
 
     def add_lines(self, version_id, parents, lines):
         """Add a version and save the weave."""

=== modified file 'bzrlib/weavefile.py'
--- bzrlib/weavefile.py	2006-03-01 05:43:15 +0000
+++ bzrlib/weavefile.py	2006-03-02 06:23:15 +0000
@@ -92,16 +92,16 @@
 
 
 
-def read_weave(f, prelude=False):
+def read_weave(f):
     # FIXME: detect the weave type and dispatch
     from bzrlib.trace import mutter
     from weave import Weave
     w = Weave(getattr(f, 'name', None))
-    _read_weave_v5(f, w, prelude=prelude)
+    _read_weave_v5(f, w)
     return w
 
 
-def _read_weave_v5(f, w, prelude=False):
+def _read_weave_v5(f, w):
     """Private helper routine to read a weave format 5 file into memory.
     
     This is only to be used by read_weave and WeaveFile.__init__.
@@ -141,9 +141,6 @@
         else:
             raise WeaveFormatError('unexpected line %r' % l)
 
-    if prelude:
-        return w
-
     while True:
         l = f.readline()
         if l == 'W\n':

